// Copyright (c) 2022-2023 VMware Carbon Black, all rights reserved

package hints

import (
	"time"

	"gitlab.bit9.local/psc/ems/hinting/internal/app/models"
	"gitlab.bit9.local/psc/ems/hinting/internal/app/models/types"
	"gitlab.bit9.local/psc/ems/hinting/pkg/enums/deploymenttype"
)

// NOTE: hint generation is limited to the second, see "internal/hints/hint_generator.go:UpdateHints()"

//goland:noinspection GoSnakeCaseUsage,GoUnusedGlobalVariable
var (
	NOW                = time.Now()
	LAST_UPDATE_TIME   = Yesterday()
	TOMORROW_TIMESTAMP = Tomorrow()
	EARLIER_TIMESTAMP  = DayBeforeYesterday()
)

func Yesterday() time.Time {
	oneDay, _ := time.ParseDuration("24h")
	yesterday := NOW.Add(-oneDay)
	return yesterday
}

func DayBeforeYesterday() time.Time {
	twoDay, _ := time.ParseDuration("48h")
	dayBefore := NOW.Add(-twoDay)
	return dayBefore
}

func Tomorrow() time.Time {
	oneDay, _ := time.ParseDuration("24h")
	tomorrow := NOW.Add(oneDay)
	return tomorrow
}

func getMockOrg() *models.Organization {
	// orgkey: Env "local", orgId "2023"
	org := models.NewOrganizationWithoutDeploymentTypeChanges("7ZP4H5XR", false)
	return org
}

func getWorstCaseMockOrg() *models.Organization {
	// orgkey: Env "local", orgId "2023"
	worstOrg := models.NewOrganizationWithoutDeploymentTypeChanges("7ZP4H5XR", false)
	universalChanges := types.NewChangesetAtTime(NOW)
	endpointChanges := types.NewChangesetAtTime(NOW)
	worstOrg.ChangeTimestamps = universalChanges
	worstOrg.DeploymentTypeChangeTimestamps[deploymenttype.ENDPOINT] = endpointChanges
	return worstOrg
}

func getMockDevice() *models.Device {
	var nowPointer *time.Time = nil
	device := models.NewDeviceWindowsEndpoint(123, "7ZP4H5XR", nowPointer)
	return device
}

func getMockDeviceGroup() *models.DeviceGroup {
	deviceGroup := models.NewDeviceGroup(1337, "7ZP4H5XR", nil, nil)
	return deviceGroup
}

func getWorstMockDeviceGroup() *models.DeviceGroup {
	deviceGroup := models.NewDeviceGroup(1337, "7ZP4H5XR", &NOW, &NOW)
	return deviceGroup
}

func getMockEnv() *models.Environment {
	env := models.NewEnv("local", nil, nil)
	return env
}

// TODO: Make actual worst case
func getWorstMockEnv() *models.Environment {
	env := models.NewEnv("local", &NOW, &NOW)
	return env
}

func getMockHintRequest(lastUpdateTime, lastLiveQueryTime *time.Time) *HintRequest {
	return NewHintRequest(
		getMockEnv(),
		getMockOrg(),
		getMockDeviceGroup(),
		getMockDevice(),
		BASEMESSAGE.PSC_VERSION,
		lastUpdateTime,
		lastLiveQueryTime,
		false)
}

func getWorstCaseHintRequest(lastUpdateTime, lastLiveQueryTime *time.Time) *HintRequest {
	return NewHintRequest(
		getWorstMockEnv(),
		getWorstCaseMockOrg(),
		getWorstMockDeviceGroup(),
		getMockDevice(),
		BASEMESSAGE.PSC_VERSION,
		lastUpdateTime,
		lastLiveQueryTime,
		false)
}
